using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public static class GameObjectExtensions
{
    /// <summary>
    /// Returns all child objects whose name contains query.
    /// </summary>
    public static IEnumerable<GameObject> FindChildren(this GameObject component, string query)
    {
        var children = new List<GameObject>();
        foreach (Transform t in component.transform)
        {
            if (t.name.Contains(query))
                children.Add(t.gameObject);
            children.AddRange(FindChildren(t.gameObject, query));
        }
        return children;
    }

    /// <summary>
    /// Find the child whose name exactly matches query.
    /// </summary>
    public static Transform FindTransform(this GameObject component, string query)
    {
        foreach (Transform t in component.transform)
        {
            if (t.name == query)
                return t;
            else
            {
                var c = FindTransform(t.gameObject, query);
                if (c != null)
                    return c;
            }
        }
        return null;
    }

    /// <summary>
    /// Finds the child who's name exactly manages query.
    /// </summary>
    public static GameObject FindChild(this GameObject component, string query)
    {
        foreach (Transform t in component.transform)
        {
            if (t.name == query)
                return t.gameObject;
            else
            {
                var c = FindChild(t.gameObject, query);
                if (c != null)
                    return c;
            }
        }
        return null;
    }


    /// <summary>
    /// Gets the entire bounds of anything rendered under this game object.
    /// </summary>
    public static Bounds GetBounds(this GameObject g)
    {
        Bounds b = new Bounds(Vector3.zero, Vector3.zero);
        var first = true;
        foreach (var i in g.GetComponentsInChildren<Renderer>())
        {
            if (first)
                b = i.bounds;
            else
                b.Encapsulate(i.bounds);
            first = false;
        }
        return b;
    }

    /// <summary>
    /// Get all direct children of this game object.
    /// </summary>
    public static GameObject[] Children(this GameObject g)
    {
        var list = new List<GameObject>();
        foreach (Transform t in g.transform)
        {
            list.Add(t.gameObject);
        }
        return list.ToArray();
    }

    /// <summary>
    /// Set this gameobject and all children to a layer.
    /// </summary>
    public static void SetLayer(this GameObject g, int layer)
    {
        g.layer = layer;
        foreach (Transform i in g.transform)
            SetLayer(i.gameObject, layer);
    }

    /// <summary>
    /// Get a component from a game object, and add it if it is missing.
    /// </summary>
    public static T DefaultComponent<T>(this GameObject g) where T : Component
    {
        T c = g.GetComponent<T>();
        if (c == null)
            c = g.AddComponent<T>();
        return c;
    }
}
