using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public static class TransformExtensions
{
    /// <summary>
    /// Resets the local position to (0,0,0).
    /// </summary>
    public static void ResetPosition(this Transform transform)
    {
        transform.localPosition = Vector3.zero;
    }

    /// <summary>
    /// Resets the local rotation to (0,0,0,0).
    /// </summary>
    public static void ResetRotation(this Transform transform)
    {
        transform.localRotation = Quaternion.identity;
    }

    /// <summary>
    /// Resets the local scale to (1,1,1).
    /// </summary>
    public static void ResetScale(this Transform transform)
    {
        transform.localScale = Vector3.one;
    }

    /// <summary>
    /// Sets the local transform's x value.
    /// </summary>
    public static void SetX(this Transform transform, float x)
    {
        Vector3 result = transform.localPosition;
        result.x = x;
        transform.localPosition = result;
    }

    /// <summary>
    /// Sets the local transform's y value.
    /// </summary>
    public static void SetY(this Transform transform, float y)
    {
        Vector3 result = transform.localPosition;
        result.y = y;
        transform.localPosition = result;
    }

    /// <summary>
    /// Sets the local transform's z value.
    /// </summary>
    public static void SetZ(this Transform transform, float z)
    {
        Vector3 result = transform.localPosition;
        result.z = z;
        transform.localPosition = result;
    }

    /// <summary>
    /// Sets current object and all children to a layer.
    /// </summary>
    public static void SetLayer(this Transform transform, int layer)
    {
        transform.gameObject.layer = layer;

        Transform[] childGOs = transform.GetComponentsInChildren<Transform>(true);

        for (int i = 0; i < childGOs.Length; ++i)
        {
            childGOs[i].gameObject.layer = layer;
        }
    }

    /// <summary>
    /// Returns a screen rect that is centred over this transform.
    /// </summary>
    public static Rect ScreenRect(this Transform tx, float width, float height)
    {
        var pos = Camera.main.WorldToScreenPoint(tx.position);
        pos.y = Screen.height - pos.y;
        var r = new Rect(pos.x - (width / 2), pos.y - (height / 2), width, height);
        return r;
    }

    /// <summary>
    /// Returns the closest transform from a list of transforms.
    /// </summary>
    public static Transform FindClosest(this Transform tx, IList<Transform> list)
    {
        Transform t = null;
        var closest = -0f;
        foreach (var i in list)
        {
            var m = (i.position - tx.position).sqrMagnitude;
            if (t == null)
            {
                closest = m;
                t = i;
            }
            else
            {
                if (m < closest)
                {
                    t = i;
                    closest = m;
                }
            }
        }
        return t;
    }

    /// <summary>
    /// Returnts the furthest transform from a list of transforms.
    /// </summary>
    public static Transform FindFurthest(this Transform tx, IList<Transform> list)
    {
        Transform t = null;
        var furthest = -0f;
        foreach (var i in list)
        {
            var m = (i.position - tx.position).sqrMagnitude;
            if (t == null)
            {
                furthest = m;
                t = i;
            }
            else
            {
                if (m > furthest)
                {
                    t = i;
                    furthest = m;
                }
            }
        }
        return t;
    }
}
