using UnityEngine;
using System.Collections;

public static class MathTools
{
    // this function assumes the vectors are being placed on the xy-plane.
    public static float AngleBetween(Vector3 a, Vector3 b)
    {
        return Mathf.Acos(Vector3.Dot(a, b) / (a.magnitude * b.magnitude));
    }

    // this function assumes the vectors are being placed on the xy-plane.
    public static float AngleBetweenSigned(Vector3 a, Vector3 b)
    {
        return AngleBetween(a, b) * Mathf.Sign(Vector3.Cross(a, b).z);
    }

    public static int Wrap(int val, int max)
    {
        return (val < 0) ? max + val : val % max;
    }

    // padding should be a value from [0..1] defining how close to the edge t
    public static Vector3 ScreenSpaceForVector(Vector3 vec, float padding)
    {
        Vector3 result = Vector3.zero;

        float maxHeight = Screen.height * padding * 0.5f;
        float maxWidth = Screen.width * padding * 0.5f;

        float absx = Mathf.Abs(vec.x);
        float absy = Mathf.Abs(vec.y);

        result.y = (vec.x == 0 ? maxHeight : Mathf.Min(maxHeight, (absy / absx) * maxWidth)) * ((vec.y >= 0) ? 1f : -1f);
        result.x = (vec.y == 0 ? maxWidth : Mathf.Min(maxWidth, (absx / absy) * maxHeight)) * ((vec.x >= 0) ? 1f : -1f);

        return result;
    }

    // returns a smoothed value from 0 to 1 for any x = [0,1]
    public static float SmoothStep(float x)
    {
        // y = x� * (3 - 2x)
        x = Mathf.Clamp01(x);
        return x * x * (3f - (2f * x));
    }

    // returns a quadratic out value from 0 to 1 for any x = [0,1]
    public static float QuadOutStep(float x)
    {
        return x * (x - 2f);
    }

    // returns a cosine wave that ranges from 0 to 1 for any x
    public static float CosineWave(float x)
    {
        // y = (1 - cos(x*pi))/2
        return (1 - Mathf.Cos(x * Mathf.PI)) / 2f;
    }

    public static bool IsPowerOfTwo(ulong x)
    {
        return (x != 0) && ((x & (x - 1)) == 0);
    }

    // some tools grabbed from: http://wiki.unity3d.com/index.php/Mathfx

    public static float Hermite(float start, float end, float value)
    {
        return Mathf.Lerp(start, end, value * value * (3.0f - 2.0f * value));
    }

    public static float Sinerp(float start, float end, float value)
    {
        return Mathf.Lerp(start, end, Mathf.Sin(value * Mathf.PI * 0.5f));
    }

    public static float Coserp(float start, float end, float value)
    {
        return Mathf.Lerp(start, end, 1.0f - Mathf.Cos(value * Mathf.PI * 0.5f));
    }

    public static float Berp(float start, float end, float value)
    {
        value = Mathf.Clamp01(value);
        value = (Mathf.Sin(value * Mathf.PI * (0.2f + 2.5f * value * value * value)) * Mathf.Pow(1f - value, 2.2f) + value) * (1f + (1.2f * (1f - value)));
        return start + (end - start) * value;
    }

    public static Vector3 NearestPoint(Vector3 lineStart, Vector3 lineEnd, Vector3 point)
    {
        Vector3 lineDirection = Vector3.Normalize(lineEnd - lineStart);
        float closestPoint = Vector3.Dot((point - lineStart), lineDirection) / Vector3.Dot(lineDirection, lineDirection);
        return lineStart + (closestPoint * lineDirection);
    }

    public static Vector3 NearestPointStrict(Vector3 lineStart, Vector3 lineEnd, Vector3 point)
    {
        Vector3 fullDirection = lineEnd - lineStart;
        Vector3 lineDirection = Vector3.Normalize(fullDirection);
        float closestPoint = Vector3.Dot((point - lineStart), lineDirection) / Vector3.Dot(lineDirection, lineDirection);
        return lineStart + (Mathf.Clamp(closestPoint, 0.0f, Vector3.Magnitude(fullDirection)) * lineDirection);
    }
    public static float Bounce(float x)
    {
        return Mathf.Abs(Mathf.Sin(6.28f * (x + 1f) * (x + 1f)) * (1f - x));
    }

    // test for value that is near specified float (due to floating point inprecision)
    // all thanks to Opless for this!
    public static bool Approx(float val, float about, float range)
    {
        return ((Mathf.Abs(val - about) < range));
    }

    // test if a Vector3 is close to another Vector3 (due to floating point inprecision)
    // compares the square of the distance to the square of the range as this 
    // avoids calculating a square root which is much slower than squaring the range
    public static bool Approx(Vector3 val, Vector3 about, float range)
    {
        return ((val - about).sqrMagnitude < range * range);
    }

    /*
      * CLerp - Circular Lerp - is like lerp but handles the wraparound from 0 to 360.
      * This is useful when interpolating eulerAngles and the object
      * crosses the 0/360 boundary.  The standard Lerp function causes the object
      * to rotate in the wrong direction and looks stupid. Clerp fixes that.
      */
    public static float Clerp(float start, float end, float value)
    {
        float min = 0.0f;
        float max = 360.0f;
        float half = Mathf.Abs((max - min) / 2.0f);//half the distance between min and max
        float retval = 0.0f;
        float diff = 0.0f;

        if ((end - start) < -half)
        {
            diff = ((max - start) + end) * value;
            retval = start + diff;
        }
        else if ((end - start) > half)
        {
            diff = -((max - end) + start) * value;
            retval = start + diff;
        }
        else
            retval = start + (end - start) * value;

        // Debug.Log("Start: "  + start + "   End: " + end + "  Value: " + value + "  Half: " + half + "  Diff: " + diff + "  Retval: " + retval);
        return retval;
    }

    /* mapping values from one range to the next */
    // thx: http://blog.prime31.com/simple-value-mapping/
    public static float Map01(float value, float min, float max)
    {
        return (value - min) / (max - min);
    }

    public static float Map(float value, float inputMin, float inputMax, float outputMin, float outputMax)
    {
        return outputMin + (value - inputMin) * (outputMax - outputMin) / (inputMax - inputMin);
    }

/* Not implemented:
 * 
 * for tile-based games:
 * IndexFor(x,y) = x + y * width
 * PositionFor(i) = (i % width,floor(i / width))
 * 
 * http://en.wikipedia.org/wiki/Fast_inverse_square_root
 * 
 * http://dinodini.wordpress.com/2010/04/05/normalized-tunable-sigmoid-functions/
 * 
 * http://en.wikipedia.org/wiki/Perlin_noise
 * 
 * http://www.altdevblogaday.com/2011/03/21/moving-beyond-the-linear-bezier/
 * 
 * http://devmaster.net/forums/topic/9796-ease-in-ease-out-algorithm/
 * 
 * suggestions taken from: http://www.reddit.com/r/gamedev/comments/iomn3/what_kinds_of_equations_do_you_find_useful_or/
 */
 
}
