﻿using System.Collections;
using System.Collections.Generic;
using System;

/// <summary>
/// Since Unity doesn't support some LINQ functions on iOS, we're going to do our own non-emitting 
/// versions because they're so convenient.
/// </summary>
public static class SafeLinq
{
	public static bool Any<T> (this IEnumerable<T> enumerable, Func<T, bool> predicate)
	{
		// Can't rely on FirstOrDefault because default may be contained in the IEnumerable
		foreach (var item in enumerable) {
			if (predicate == null || predicate (item)) {
				return true;
			}
		}

		return false;
	}

	// Technically not a LINQ function, but handy to have
	public static bool Contains<T> (this IEnumerable<T> enumerable, T instance)
	{
		foreach (var item in enumerable) {
			if (item.Equals (instance)) {
				return true;
			}
		}
		return false;
	}

	public static int Count<T> (this IEnumerable<T> enumerable, Func<T, bool> predicate = null)
	{
		int toReturn = 0;

		foreach (var item in enumerable) {
			if (predicate == null || predicate (item)) {
				toReturn++;
			}
		}

		return toReturn;
	}

	public static int Sum<T> (this IEnumerable<T> enumerable, Func<T, int> predicate)
	{
		int toReturn = 0;

		foreach (var item in enumerable) {
			toReturn += predicate (item);
		}

		return toReturn;
	}

	public static IEnumerable<T> Except<T> (this IEnumerable<T> enumerable, T toExclude)
	{
		foreach (var item in enumerable) {				
			if (!toExclude.Equals (item)) {
				yield return item;
			}
		}
	}

	public static IEnumerable<T> Except<T> (this IEnumerable<T> enumerable, IEnumerable<T> toExclude)
	{
		foreach (var item in enumerable) {
			bool isContained = false;

			foreach (var otherItem in toExclude) {
				if (otherItem.Equals (item)) {
					isContained = true;
					break;
				}
			}

			if (!isContained) {
				yield return item;
			}
		}
	}

	public static T FirstOrDefault<T> (this IEnumerable<T> enumerable, Func<T, bool> predicate = null)
	{
		foreach (var item in enumerable) {
			if (predicate == null || predicate (item)) {
				return item;
			}
		}

		return default(T);
	}

	public static int Max<T> (this IEnumerable<T> enumerable, Func<T, int> predicate)
	{
		int? maxValue = null;

		foreach (var item in enumerable) {
			int valueForItem = predicate (item);

			if (maxValue.HasValue == false || valueForItem > maxValue.Value) {
				maxValue = valueForItem;
			}
		}

		if (maxValue.HasValue == false) {
			return int.MinValue;
		}

		return maxValue.Value;            
	}

	public static int Min<T> (this IEnumerable<T> enumerable, Func<T, int> predicate)
	{
		int? minValue = null;

		foreach (var item in enumerable) {
			int valueForItem = predicate (item);

			if (minValue.HasValue == false || valueForItem < minValue.Value) {
				minValue = valueForItem;
			}
		}

		if (minValue.HasValue == false) {
			return int.MaxValue;
		}

		return minValue.Value;  
	}

	public static IEnumerable<TResult> Select<TSource, TResult> (this IEnumerable<TSource> enumerable, Func<TSource, TResult> selectFunc)
	{
		foreach (var item in enumerable) {
			yield return selectFunc (item);
		}
	}

	public static IEnumerable<TResult> SelectMany<TSource, TResult> (this IEnumerable<TSource> enumerable, Func<TSource, IEnumerable<TResult>> selectFunc)
	{
		foreach (var item in enumerable) {
			var subEnumerable = selectFunc (item);
			foreach (var subitem in subEnumerable) {
				yield return subitem;
			}
		}
	}

	public static IEnumerable<T> Skip<T> (this IEnumerable<T> enumerable, int count)
	{
		int numberSkippedSoFar = 0;

		foreach (var item in enumerable) {
			if (numberSkippedSoFar < count) {
				numberSkippedSoFar++;
			} else {
				yield return item;
			}
		}
                
	}

	public static IEnumerable<T> Take<T> (this IEnumerable<T> enumerable, int count)
	{
		int numberTakenSoFar = 0;
		foreach (var item in enumerable) {
			if (numberTakenSoFar >= count) {
				break;
			} else {
				numberTakenSoFar++;
				yield return item;
			}
		}
	}

	public static List<T> ToList<T> (this IEnumerable<T> enumerable)
	{
		List<T> toReturn = new List<T> ();

		toReturn.AddRange (enumerable);

		return toReturn;
	}

	public static IEnumerable<T> Where<T> (this IEnumerable<T> enumerable, Func<T, bool> predicate)
	{
		foreach (var item in enumerable) {
			if (predicate (item)) {
				yield return item;
			}
		}
	}

}